/* $Header: /fridge/cvs/cgi.targeted/targeted-nav.c,v 1.14 2001/12/11 03:54:06 justins Exp $ */
/*
   
   targeted-nav.c             Copyright(c) 2001-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Functions for navigation bars
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


/* Include files */
#include <targeted-nav.h>
#include <targeted-list.h>
#include <targeted-util.h>
#include <targeted-cgi.h>
#include <libjtree.h>


static void _print_path_info(const config *c, const char *r_path, const char *msg, bool thumb, bool toolbox) {
/* Print information on this directory link */

   bintree *cgi;
   char buf[0x1000];

   if(targeted_index_present(c, r_path)) {
      printf("<a href=\"%s%s\">", stra(c->dir_prefix_uri), r_path);
   } else {
      cgi = clone_cgi_arguments(c->env.query_args);
      set_cgi_argument(cgi, "mode", thumb ? "thumb" : "view");
      if(toolbox) {
         unset_cgi_argument(cgi, "toolbox");
      } else {
         set_cgi_argument(cgi, "toolbox", "0");
      }
      set_cgi_argument(cgi, "path", r_path);
      construct_cgi_url(c, cgi, buf, sizeof(buf));
      printf("<a href=\"%s\">", buf);
      bintree_release(&cgi);
   }
   printf("%s</a>", msg);

}


static void _print_named_path(const config *c, const char *parent, const char *r_path, bool thumb, bool toolbox) {
/* Print a path info URL (as above), using the format
   prefix + path for the display string.  A URL is only
   displayed if the r_path is not NULL, and is not a
   directory like "." or "..".  */

   char path[0x1000];
   
   if(parent != NULL && r_path != NULL && !strequal(r_path, ".") && !strequal(r_path, "..")) {
      qualify_rel_path_file(c, path, sizeof(path), parent, r_path);
      _print_path_info(c, path, r_path, thumb, toolbox);
   } else {
      printf("None");
   }

}


static void _print_parent_path(const config *c, const char *parent, bool thumb, bool toolbox) {
/* Displays a URL for the parent path */

   char buf[0x1000];
   const char *p;
   const char *p1;
   const char *p2;

   if(parent != NULL) {
      p = parent;
      p1 = p;
      p2 = p;
      while(*p != '\0') {
         if(*p == '/' && *(p + 1) != '\0') p1 = p + 1;
         else if(*p == '/') p2 = p + 1;
         ++p;
      }
      if(p2 <= p1 + 1) {
         _print_path_info(c, parent, "root", thumb, toolbox);
      } else {
         strcopybn(buf, p1, p2 - p1, sizeof(buf));
         _print_path_info(c, parent, buf, thumb, toolbox);
      }
   } else {
      printf("None");
   }

}


static void _print_children_paths(const config *c, const char *r_path, const char **children, int num, bool overflow, bool thumb, bool toolbox) {
/* Displays URLs of all the listed children */

   int i;
   int found = 0;
   
   for(i = 0; i < num; ++i) {
      if(children[i] != NULL) {
         if(found > 0) printf(", ");
         ++found;
         _print_named_path(c, r_path, children[i], thumb, toolbox);
      }
   }
   if(found == 0) printf("None");
   if(overflow) printf(", ...");

}


void targeted_nav_bar(const config *c, const char *r_path, bool thumb, bool toolbox) {
/* Code for displaying the navigation bars */

   char buf[0x1000];
   char d_name[0x1000];
   char parent[0x1000];
   bintree *parent_list = NULL;
   bintree *current_list = NULL;
   bintree *cgi = NULL;
   const char *p;
   const char *p1;
   const char *p2;
   const char *prev = NULL;
   const char *next = NULL;
   const char *children[] = { NULL, NULL, NULL };
   bool children_overflow = false;
   bool toolbox_active;
   int i;

   assert(c != NULL);
   assert(r_path != NULL);

   /* Make sure directory name is valid */
   if(!validate_directory_name(r_path)) return;

   /* Check if the toolbox is active or not */
   toolbox_active = atoi(lookup_cgi_argument(c->env.query_args, "toolbox", "1"));
 
   /* Get the directory name/d_name */
   p1 = r_path + strlenn(r_path) - 2;
   while(p1 > r_path && *p1 != '/') --p1;
   if(*p1 == '/') ++p1;
   p2 = p1;
   while(*p2 != '\0' && *p2 != '/') ++p2;
   strcopynb(d_name, p1, p2 - p1, sizeof(d_name));
   
   /* Get my directory listing, and find my first child */
   current_list = targeted_list_dir(c, r_path, is_directory);
   if(current_list != NULL) {
      children_overflow = targeted_children(c, children, sizeof(children) / sizeof(const char *), current_list);
   }

   /* Try to find information on the parent */
   printf("<p>");
   strcopyb(parent, "", sizeof(parent));
   if(compute_parent_directory(parent, sizeof(parent), r_path)) {
      /* Get a directory listing on the parent */
      parent_list = targeted_list_dir(c, parent, is_directory);
      if(parent_list != NULL) {
         /* Get prior/next entries, if they exist */
         prev = targeted_prev(c, parent_list, d_name);
         next = targeted_next(c, parent_list, d_name);
   
         /* Print prev links */
         printf("&lt;- Prev: ");
         _print_named_path(c, parent, prev, thumb, toolbox_active);
         printf(" | ");

         /* Print parent links */
         printf("Parent: ");
         _print_parent_path(c, parent, thumb, toolbox_active);
         printf(" | ");
         
         /* Print descent link (if any) */
         printf("Down: ");
         _print_children_paths(c, r_path, children, sizeof(children) / sizeof(const char *), children_overflow, thumb, toolbox_active);
         printf(" | ");
      
         /* Print next links */
         printf("Next: ");
         _print_named_path(c, parent, next, thumb, toolbox_active);
         printf(" -&gt;<br>\n");
      } /* Did we get directory list on parent? */
   } /* Do we have parent info? */
      
   /* Thumbnail toggle */
   if(thumb) {
      _print_path_info(c, r_path, "Switch to view <i>without</i> thumbnails", false, toolbox_active);
   } else {
      _print_path_info(c, r_path, "Switch to view <i>with</i> thumbnails", true, toolbox_active);
   }
   printf(" | ");
   
   /* Toolbox toggle */
   if(toolbox_active) {
      _print_path_info(c, r_path, "Turn <i>off</i> toolbox", thumb, false);
   } else {
      _print_path_info(c, r_path, "Turn <i>on</i> toolbox", thumb, true);
   }
   if(thumb) {
      printf(" | Number of columns: ");
      cgi = clone_cgi_arguments(c->env.query_args);
      for(i = 2; i <= 8; ++i) {
         set_cgi_argument_int(cgi, "numcols", i);
         /* If using an old-format URL, the path wouldn't have been set */
         set_cgi_argument(cgi, "path", r_path);
         set_cgi_argument(cgi, "mode", thumb ? "thumb" : "view");
         construct_cgi_url(c, cgi, buf, sizeof(buf));
         printf("<a href=\"%s\">%d</a> ", buf, i);
      }
      bintree_release(&cgi);
   }
   printf("</p>\n"); 
   
   /* Print toolbox if requested */
   if(c->use_toolbox && toolbox && toolbox_active) {
      printf("<div class=toolbox>\n");

      /* Figure out the current directory's basename */
      p = r_path;
      p1 = p;
      p2 = p;
      while(*p != '\0') {
         if(*p == '/' && *(p + 1) != '\0') p1 = p + 1;
         else if(*p == '/') p2 = p + 1;
         ++p;
      }
      strcopynb(buf, p1, p2 - p1, sizeof(buf));
      printf("<div><span>Directory:</span><span>%s</span></div>\n", buf);
      
      /* Print thumbnail links */
      printf("<div><span>Thumbnails:</span><span>");
      if(thumb) {
         _print_path_info(c, r_path, "Turn OFF", false, toolbox_active);
      } else {
         _print_path_info(c, r_path, "Turn ON", true, toolbox_active);
      }
      printf("</span></div>\n");
      
      /* Print parent directory */
      printf("<div><span>Parent:</span><span>");
      _print_parent_path(c, parent, thumb, toolbox_active);
      printf("</span></div>\n");
      
      /* Print descendant(s) */
      for(i = 0; i < sizeof(children) / sizeof(const char *); ++i) {
         if(children[i] != NULL) {
            printf("<div%s><span>%s</span><span>", i == 0 ? "" : " class=noline", i == 0 ? "Down:" : "");
            _print_named_path(c, r_path, children[i], thumb, toolbox_active);
            printf("</span></div>\n");
         }
      }
      if(children_overflow) {
         printf("<div class=noline><span></span><span>...</span></div>\n");
      }
      
      /* Print previous directory */
      printf("<div><span>Prev:</span><span>");
      _print_named_path(c, parent, prev, thumb, toolbox_active);
      printf("</span></div>\n");
      
      /* Print next directory */
      printf("<div><span>Next:</span><span>");
      _print_named_path(c, parent, next, thumb, toolbox_active);
      printf("</span></div>\n");
      
      /* Print final (clearing) line */
      printf("<div class=\"noline clear\"></div>\n");
      printf("</div>\n");
   }

   /* release the directory list */
   bintree_release(&parent_list);
   bintree_release(&current_list);

}


void targeted_header(const config *c, const char *path, bool thumb) {

   char buf[0x1000];
   FILE *f;

   printf("<h1>%s %s</h1>\n", thumb ? "Thumbnails for" : "Index of", path);
   
   /* Validate the path */
   if(validate_directory_name(path)) {
      if(targeted_check_access(c, path) == ACCESS_OK) {
         /* Check to see if .HEADER file exists */
         qualify_full_path_file(c, buf, sizeof(buf), path, stra(c->header_name));
         if((f = fopen(buf, "r")) != NULL) {
            /* .HEADER file exists; dump it */
            printf("<p>");
            printf("Directory information: ");
            while(fgets(buf, sizeof(buf), f) != NULL) {
               printf("%s", buf);
            }
            fclose(f);
            printf("</p>\n");
         }
      }
   }

   /* Print out the navbar */
   targeted_nav_bar(c, path, thumb, false);
   printf("<hr>\n\n");

}


void targeted_footer(const config *c, const char *path, bool thumb) {

   printf("<hr>\n\n");
   targeted_nav_bar(c, path, thumb, true);
   
}


