/* $Header: /fridge/cvs/cgi.targeted/targeted-thumb.c,v 1.13 2001/12/11 03:54:06 justins Exp $ */
/*
   
   targeted-thumb.c           Copyright(c) 2001-1998 Justin David Smith
   justins(at)chaos2.org      http://chaos2.org/
    
   Functions for managing thumbnail display mode
    

   This program is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as published by 
   the Free Software Foundation, version 2 of the License ONLY.

   This program is distributed in the hope that it will be useful, 
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation, 
   Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include <targeted-util.h>
#include <targeted-thumb.h>
#include <targeted-config.h>
#include <targeted-list.h>
#include <targeted-view.h>
#include <targeted-image.h>
#include <targeted-nav.h>
#include <targeted-log.h>
#include <targeted-cgi.h>


static bool _string_ends_in(const char *s, const char *pat) {
/* Return true if string s has corresponding suffix pat. */

   int plen = strlenn(pat);
   int slen = strlenn(s);
   int pofs;
   
   if(plen > slen) return(false);
   pofs = slen - plen;
   return(strequaln(s + pofs, pat, plen));

}


bool is_image_file(const config *c, const char *r_path, const char *d_name) {
/* Returns true if d_name is a valid image file name */

   if(!not_is_directory(c, r_path, d_name)) return(false);
   if(_string_ends_in(d_name, ".jpg"))  return(true);
   if(_string_ends_in(d_name, ".jpeg")) return(true);
   if(_string_ends_in(d_name, ".gif"))  return(true);
   if(_string_ends_in(d_name, ".png"))  return(true);
   return(false);

}


typedef struct _print_data {
   const config *c;
   const char *r_path;
   int columns;
   int column;
   int count;
} print_data;


static bool _print_image(const void *_data, void *_params) {
/* Prints out a table cell containing an image thumbnail */

   bintree *cgi;
   image_desc *desc;
   const char *d_name = (const char *)_data;
   print_data *params = (print_data *)_params;
   char rfile[0x1000];
   char buf[0x1000];
   const char *usefile;

   qualify_rel_path_file(params->c, rfile, sizeof(rfile), params->r_path, d_name);

   /* Image file (we hope!) */ 
   cgi = clone_cgi_arguments(params->c->env.query_args);
   set_cgi_argument(cgi, "mode", "viewimage");
   desc = new_image_desc(params->c, rfile);
   if(desc != NULL) {
      if(params->column == 0) printf("<tr class=listdir>");
      else printf("   ");
      if(!strequal(stra(desc->thumb), "")) usefile = stra(desc->thumb);
      else usefile = rfile;
      printf("<td class=image>%s<br>(%dx%d, ", d_name, desc->width, desc->height);
      print_file_size(stdout, desc->size);
      
      set_cgi_argument(cgi, "path", rfile);
      construct_cgi_url(params->c, cgi, buf, sizeof(buf));
      printf(")<br><a href=\"%s%s\"><img class=border src=\"%s\" width=%d height=%d alt=\"Thumbnail for %s\"></a>", stra(params->c->dir_prefix_uri), buf, usefile, desc->thumbwidth, desc->thumbheight, rfile);
      if(desc->width != desc->thumbwidth || desc->height != desc->thumbheight) {
         printf("<br>(rescaled to %dx%d, ", desc->thumbwidth, desc->thumbheight);
         print_file_size(stdout, desc->thumbsize);
         printf(")");
      }
      printf("</td>");
      params->column = (params->column + 1) % params->columns;
      if(params->column == 0) printf("</tr>");
      printf("\n");
      ++params->count;
      free_image_desc(&desc);
   }
   bintree_release(&cgi);
   return(true);

}


static bool _display_thumbnails(const config *c, const char *r_path) {
/* Display the thumbnails */

   bintree *imglist;
   print_data params;
   char buf[0x1000];
   
   /* Make sure a directory list is allowed here */
   if(!targeted_view_okay(c, r_path, true)) return(false);
   
   /* Check for existence of the path; open it */
   imglist  = targeted_list_dir(c, r_path, is_image_file);
   if(imglist == NULL) {
      targeted_log_error(c, "unable to open directory to view thumbnails");
      printf("<p class=error><b>targeted error:</b> Invalid directory specified, or you do not have access rights to this directory.</p>\n");
      targeted_warning();
      return(false);
   } 

   /* Process image or directory files in this directory */
   printf("<h2>Thumbnails</h2>\n");
   printf("<table class=listing summary=\"Listing of thumbnails for images in this directory\">");

   /* Determine the number of columns to display */
   sbprintf(buf, sizeof(buf), "%d", c->thumbnail_cols);
   params.columns = atoi(lookup_cgi_argument(c->env.query_args, "numcols", buf));
   if(params.columns < 1) params.columns = 1;

   /* Walk through the images in alpha order */
   params.c = c;
   params.r_path = r_path;
   params.column = 0;
   params.count = 0;
   bintree_walk(imglist, _print_image, &params);
   if(params.count == 0) {
      printf("<tr class=error><td>No images present in this directory.</td></tr>\n");
   } else if(params.column != 0) {
      printf("</tr>\n");
   }
   printf("</table>\n\n");
   
   /* Release trees */
   bintree_release(&imglist);
   return(true);

}


static void _targeted_thumb(const config *c, const char *in_r_path) {
/* Display thumbnails */

   stringa *r_path;
   bool display_ok;

   /* Prepare the page */
   r_path = rewrite_directory_name(in_r_path);
   targeted_begin(c, stra(r_path));
   targeted_header(c, stra(r_path), true);

   /* Grab thumbnail files */
   display_ok = _display_thumbnails(c, stra(r_path));

   /* Emit the normal listing */
   if(display_ok) {
      targeted_view_directory(c, "thumb", stra(r_path));
   }

   /* Footer */
   targeted_footer(c, stra(r_path), true);
   targeted_end(c);
   strreleasea(&r_path);
   
}


void targeted_thumb_qstr(const config *c, const char *in_r_path) {
/* Display thumbnails (old interface) */

   assert(c != NULL);
   assert(in_r_path != NULL);
   
   _targeted_thumb(c, in_r_path);
   
}


void targeted_thumb(const config *c) {
/* Display thumbnails */

   const char *path;

   assert(c != NULL);
   
   path = lookup_cgi_argument(c->env.query_args, "path", "");
   _targeted_thumb(c, path);
   
}
